@@ -2124,20 +2124,7 @@ int orinoco_init(struct orinoco_private *priv)
 }
 EXPORT_SYMBOL(orinoco_init);
 
-#if LINUX_VERSION_IS_LESS(4,10,0)
-static int __change_mtu(struct net_device *ndev, int new_mtu){
-	if (new_mtu < ORINOCO_MIN_MTU || new_mtu > ORINOCO_MAX_MTU)
-		return -EINVAL;
-	ndev->mtu = new_mtu;
-	return 0;
-}
-#endif
-
 static const struct net_device_ops orinoco_netdev_ops = {
-#if LINUX_VERSION_IS_LESS(4,10,0)
-	.ndo_change_mtu = __change_mtu,
-#endif
-	
 	.ndo_open		= orinoco_open,
 	.ndo_stop		= orinoco_stop,
 	.ndo_start_xmit		= orinoco_xmit,
@@ -2292,17 +2279,13 @@ int orinoco_if_add(struct orinoco_private *priv,
 
 	netif_carrier_off(dev);
 
-	memcpy(dev->dev_addr, wiphy->perm_addr, ETH_ALEN);
+	eth_hw_addr_set(dev, wiphy->perm_addr);
 
 	dev->base_addr = base_addr;
 	dev->irq = irq;
 
-#if LINUX_VERSION_IS_GEQ(4,10,0)
 	dev->min_mtu = ORINOCO_MIN_MTU;
-#endif
-#if LINUX_VERSION_IS_GEQ(4,10,0)
 	dev->max_mtu = ORINOCO_MAX_MTU;
-#endif
 
 	SET_NETDEV_DEV(dev, priv->dev);
 	ret = register_netdev(dev);
